
#include "Arduino.h"
// bras de pont phase A
#define INA_1 11 // transistor haut
#define INA_2 12 // transistor bas
// bras de pont phase B
#define INB_1 5 
#define INB_2 2 
// bras de pont phase C
#define INC_1 6 
#define INC_2 7 



// états d'un bras de pont
#define HIGH_0_LOW_1 0 // sortie à la masse (état 01)
#define HIGH_1_LOW_0 1 // sortie en PWM (état 10)
#define HIGH_0_LOW_0 2 // sortie non connectée (état 00)

uint8_t sequence_A[6] = {1,1,2,0,0,2};
uint8_t sequence_B[6] = {0,2,1,1,2,0};
uint8_t sequence_C[6] = {2,0,0,2,1,1};

uint8_t sensRot = 0;

// capteurs HALL
#define IN1 18 // PD3
#define IN2 19 // PD2
#define IN3 20 // PD1
uint8_t tableEtats[8];
uint32_t compteur;
uint32_t temps;
uint32_t period;

uint16_t icr;
uint16_t ocra,ocrb;
uint16_t diviseur[6] = {0,1,8,64,256,1024};

//#define PWM // décommenter pour activer le PWM
float periodPWM = 100;
float rapport = 0.2;
uint16_t temps_mort = 10;
                           
void config_timers(float period, float rapport) {
  // period : période du PWM en microsecondes
  cli();
  TCCR1A = (1 << COM1A1) | (1 << COM1B1) | (1 << COM1B0);
  TCCR1B = (1 << WGM13);
  TCCR3A = (1 << COM1A1) | (1 << COM1B1) | (1 << COM1B0);
  TCCR3B = (1 << WGM13);
  TCCR4A = (1 << COM1A1) | (1 << COM1B1) | (1 << COM1B0);
  TCCR4B = (1 << WGM13);

  icr = (F_CPU/1000000*period/2);
    int d = 1;
    while ((icr>0xFFFF)&&(d<5)) {
        d++;
        icr = (F_CPU/1000000*period/2/diviseur[d]);
    } 
  TCCR1B |= d;
  TCCR3B |= d;
  TCCR4B |= d;
  ocra = icr*rapport-temps_mort;
  ocrb = icr*rapport+temps_mort;
  TCNT1 = 0;
  TCNT3 = 0;
  TCNT4 = 0;
  ICR1 = icr;
  ICR3 = icr;
  ICR4 = icr;
  OCR1A = ocra;
  OCR1B = ocrb;
  OCR3A = ocra;
  OCR3B = ocrb;
  OCR4A = ocra;
  OCR4B = ocrb;
  sei();
}                         
                              
void etat_bras_A(uint8_t etat) {
#ifdef PWM
  switch (etat) {
    case HIGH_0_LOW_1:
      OCR1A = 0;
      OCR1B = 0;
      break;
    case HIGH_1_LOW_0:
      OCR1A = ocra;
      OCR1B = ocrb;
      break;
    case HIGH_0_LOW_0:
      OCR1A = 0;
      OCR1B = icr+1;
      break;
  }
  #else
    switch (etat) {
    case HIGH_0_LOW_1:
      digitalWrite(INA_1,LOW);
      digitalWrite(INA_2,HIGH);
      break;
    case HIGH_1_LOW_0:
      digitalWrite(INA_1,HIGH);
      digitalWrite(INA_2,LOW);
      break;
    case HIGH_0_LOW_0:
      digitalWrite(INA_1,LOW);
      digitalWrite(INA_2,LOW);
      break;
  }
  #endif
}

void etat_bras_B(uint8_t etat) {
#ifdef PWM
  switch (etat) {
    case HIGH_0_LOW_1:
      OCR3A = 0;
      OCR3B = 0;
      break;
    case HIGH_1_LOW_0:
      OCR3A = ocra;
      OCR3B = ocrb;
      break;
    case HIGH_0_LOW_0:
      OCR3A = 0;
      OCR3B = icr+1;
      break;
  }
#else
  switch (etat) {
    case HIGH_0_LOW_1:
      digitalWrite(INB_1,LOW);
      digitalWrite(INB_2,HIGH);
      break;
    case HIGH_1_LOW_0:
      digitalWrite(INB_1,HIGH);
      digitalWrite(INB_2,LOW);
      break;
    case HIGH_0_LOW_0:
      digitalWrite(INB_1,LOW);
      digitalWrite(INB_2,LOW);
      break;
  }
#endif
}

void etat_bras_C(uint8_t etat) {
#ifdef PWM
  switch (etat) {
    case HIGH_0_LOW_1:
      OCR4A = 0;
      OCR4B = 0;
      break;
    case HIGH_1_LOW_0:
      OCR4A = ocra;
      OCR4B = ocrb;
      break;
    case HIGH_0_LOW_0:
      OCR4A = 0;
      OCR4B = icr+1;
      break;
  }
#else
  switch (etat) {
    case HIGH_0_LOW_1:
      digitalWrite(INC_1,LOW);
      digitalWrite(INC_2,HIGH);
      break;
    case HIGH_1_LOW_0:
      digitalWrite(INC_1,HIGH);
      digitalWrite(INC_2,LOW);
      break;
    case HIGH_0_LOW_0:
      digitalWrite(INC_1,LOW);
      digitalWrite(INC_2,LOW);
      break;
  }
#endif
} 
                              
void remplir_table_etats(uint8_t sensRot) {
  if (sensRot==0) {
    tableEtats[B010] = 3;
    tableEtats[B011] = 4;
    tableEtats[B001] = 5;
    tableEtats[B101] = 0;
    tableEtats[B100] = 1;
    tableEtats[B110] = 2;
  }
  else {
    tableEtats[B010] = 0;
    tableEtats[B011] = 1;
    tableEtats[B001] = 2;
    tableEtats[B101] = 3;
    tableEtats[B100] = 4;
    tableEtats[B110] = 5;
  }

}              
                               
void change_etat() {
  uint32_t t;
  uint8_t etat_capteurs = (PIND >> 1);
  if ((etat_capteurs==0)||(etat_capteurs==0xb111)) return;
  uint8_t is = tableEtats[etat_capteurs];
  etat_bras_A(sequence_A[is]);
  etat_bras_B(sequence_B[is]);
  etat_bras_C(sequence_C[is]);

  compteur += 1;
  if (compteur==24) {
    compteur = 0;
    t = micros();
    period = t-temps;
    Serial.println(period/1e3);
    temps = t;
  }

}      
                                
void setup() {
  Serial.begin(115200);
  remplir_table_etats(sensRot);
  pinMode(INA_1,OUTPUT);
  digitalWrite(INA_1,LOW);
  pinMode(INA_2,OUTPUT);
  digitalWrite(INA_2,LOW);
  pinMode(INB_1,OUTPUT);
  digitalWrite(INB_1,LOW);
  pinMode(INB_2,OUTPUT);
  digitalWrite(INB_2,LOW);
  pinMode(INC_1,OUTPUT);
  digitalWrite(INC_1,LOW);
  pinMode(INC_2,OUTPUT);
  digitalWrite(INC_2,LOW);

  pinMode(IN1,INPUT);
  pinMode(IN2,INPUT);
  pinMode(IN3,INPUT);
 
  attachInterrupt(digitalPinToInterrupt(IN1),change_etat,CHANGE);
  attachInterrupt(digitalPinToInterrupt(IN2),change_etat,CHANGE);
  attachInterrupt(digitalPinToInterrupt(IN3),change_etat,CHANGE);
 #ifdef PWM
  config_timers(periodPWM,rapport);
 #endif
  compteur = 0;
  temps = micros();
  change_etat();
}

void loop() {


}
                                